@@ -12,7 +12,7 @@ from .base import BaseEstimator, ClassifierMixin, RegressorMixin
 from .utils import check_random_state
 from .utils.validation import check_array
 from .utils.validation import check_consistent_length
-from .utils import deprecated
+from .utils.validation import check_is_fitted
 from .utils.random import random_choice_csc
 from .utils.stats import _weighted_percentile
 from .utils.multiclass import class_distribution
@@ -25,20 +25,28 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
     This classifier is useful as a simple baseline to compare with other
     (real) classifiers. Do not use it for real problems.
 
+    Read more in the :ref:`User Guide <dummy_estimators>`.
+
     Parameters
     ----------
-    strategy : str
+    strategy : str, default="stratified"
         Strategy to use to generate predictions.
 
         * "stratified": generates predictions by respecting the training
           set's class distribution.
         * "most_frequent": always predicts the most frequent label in the
           training set.
+        * "prior": always predicts the class that maximizes the class prior
+          (like "most_frequent") and ``predict_proba`` returns the class prior.
         * "uniform": generates predictions uniformly at random.
         * "constant": always predicts a constant label that is provided by
           the user. This is useful for metrics that evaluate a non-majority
           class
 
+          .. versionadded:: 0.17
+             Dummy Classifier now supports prior fitting strategy using
+             parameter *prior*.
+
     random_state : int seed, RandomState instance, or None (default)
         The seed of the pseudo random number generator to use.
 
@@ -63,7 +71,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
     outputs_2d_ : bool,
         True if the output at fit is 2d, else false.
 
-    `sparse_output_` : bool,
+    sparse_output_ : bool,
         True if the array returned from predict is to be in sparse CSC format.
         Is automatically set to True if the input y is passed in sparse format.
 
@@ -96,7 +104,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
             Returns self.
         """
         if self.strategy not in ("most_frequent", "stratified", "uniform",
-                                 "constant"):
+                                 "constant", "prior"):
             raise ValueError("Unknown strategy type.")
 
         if self.strategy == "uniform" and sp.issparse(y):
@@ -148,8 +156,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         return self
 
     def predict(self, X):
-        """
-        Perform classification on test vectors X.
+        """Perform classification on test vectors X.
 
         Parameters
         ----------
@@ -162,8 +169,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         y : array, shape = [n_samples] or [n_samples, n_outputs]
             Predicted target values for X.
         """
-        if not hasattr(self, "classes_"):
-            raise ValueError("DummyClassifier not fitted.")
+        check_is_fitted(self, 'classes_')
 
         X = check_array(X, accept_sparse=['csr', 'csc', 'coo'])
         # numpy random_state expects Python int and not long as size argument
@@ -189,7 +195,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
 
         if self.sparse_output_:
             class_prob = None
-            if self.strategy == "most_frequent":
+            if self.strategy in ("most_frequent", "prior"):
                 classes_ = [np.array([cp.argmax()]) for cp in class_prior_]
 
             elif self.strategy == "stratified":
@@ -205,7 +211,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
             y = random_choice_csc(n_samples, classes_, class_prob,
                                   self.random_state)
         else:
-            if self.strategy == "most_frequent":
+            if self.strategy in ("most_frequent", "prior"):
                 y = np.tile([classes_[k][class_prior_[k].argmax()] for
                              k in range(self.n_outputs_)], [n_samples, 1])
 
@@ -243,8 +249,7 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
             the model, where classes are ordered arithmetically, for each
             output.
         """
-        if not hasattr(self, "classes_"):
-            raise ValueError("DummyClassifier not fitted.")
+        check_is_fitted(self, 'classes_')
 
         X = check_array(X, accept_sparse=['csr', 'csc', 'coo'])
         # numpy random_state expects Python int and not long as size argument
@@ -266,9 +271,11 @@ class DummyClassifier(BaseEstimator, ClassifierMixin):
         P = []
         for k in range(self.n_outputs_):
             if self.strategy == "most_frequent":
-                ind = np.ones(n_samples, dtype=int) * class_prior_[k].argmax()
+                ind = class_prior_[k].argmax()
                 out = np.zeros((n_samples, n_classes_[k]), dtype=np.float64)
                 out[:, ind] = 1.0
+            elif self.strategy == "prior":
+                out = np.ones((n_samples, 1)) * class_prior_[k]
 
             elif self.strategy == "stratified":
                 out = rs.multinomial(1, class_prior_[k], size=n_samples)
@@ -321,6 +328,8 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
     This regressor is useful as a simple baseline to compare with other
     (real) regressors. Do not use it for real problems.
 
+    Read more in the :ref:`User Guide <dummy_estimators>`.
+
     Parameters
     ----------
     strategy : str
@@ -360,13 +369,6 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
         self.constant = constant
         self.quantile = quantile
 
-    @property
-    @deprecated('This will be removed in version 0.17')
-    def y_mean_(self):
-        if self.strategy == 'mean':
-            return self.constant_
-        raise AttributeError
-
     def fit(self, X, y, sample_weight=None):
         """Fit the random regressor.
 
@@ -462,9 +464,7 @@ class DummyRegressor(BaseEstimator, RegressorMixin):
         y : array, shape = [n_samples]  or [n_samples, n_outputs]
             Predicted target values for X.
         """
-        if not hasattr(self, "constant_"):
-            raise ValueError("DummyRegressor not fitted.")
-
+        check_is_fitted(self, "constant_")
         X = check_array(X, accept_sparse=['csr', 'csc', 'coo'])
         n_samples = X.shape[0]
 
