@@ -14,13 +14,15 @@ from sklearn.utils.testing import assert_raises
 from sklearn.utils.testing import assert_false, assert_true
 from sklearn.utils.testing import assert_equal
 from sklearn.utils.testing import assert_raises_regexp
-from sklearn.utils.testing import assert_warns_message
+from sklearn.utils.testing import ignore_warnings
 
 from sklearn import linear_model, datasets, metrics
 from sklearn.base import clone
 from sklearn.linear_model import SGDClassifier, SGDRegressor
 from sklearn.preprocessing import LabelEncoder, scale, MinMaxScaler
 
+from sklearn.linear_model import sgd_fast
+
 
 class SparseSGDClassifier(SGDClassifier):
 
@@ -79,7 +81,7 @@ X3 = np.array([[1, 1, 0, 0, 0, 0], [1, 1, 0, 0, 0, 0],
                [0, 0, 0, 1, 0, 0], [0, 0, 0, 1, 0, 0]])
 Y3 = np.array([1, 1, 1, 1, 2, 2, 2, 2])
 
-# test sample 4 - two more or less redundent feature groups
+# test sample 4 - two more or less redundant feature groups
 X4 = np.array([[1, 0.9, 0.8, 0, 0, 0], [1, .84, .98, 0, 0, 0],
                [1, .96, .88, 0, 0, 0], [1, .91, .99, 0, 0, 0],
                [0, 0, 0, .89, .91, 1], [0, 0, 0, .79, .84, 1],
@@ -254,10 +256,17 @@ class CommonTest(object):
                                   decimal=16)
         assert_almost_equal(clf1.intercept_, average_intercept, decimal=16)
 
+    @raises(ValueError)
+    def test_sgd_bad_alpha_for_optimal_learning_rate(self):
+        # Check whether expected ValueError on bad alpha, i.e. 0
+        # since alpha is used to compute the optimal learning rate
+        self.factory(alpha=0, learning_rate="optimal")
+
 
 class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
     """Test suite for the dense representation variant of SGD"""
     factory_class = SGDClassifier
+
     def test_sgd(self):
         # Check that SGD gives any results :-)
 
@@ -372,18 +381,18 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         # Target must have at least two labels
         self.factory(alpha=0.01, n_iter=20).fit(X2, np.ones(9))
 
-    def test_partial_fit_weight_class_auto(self):
-        # partial_fit with class_weight='auto' not supported
+    def test_partial_fit_weight_class_balanced(self):
+        # partial_fit with class_weight='balanced' not supported"""
         assert_raises_regexp(ValueError,
-                             "class_weight 'auto' is not supported for "
-                             "partial_fit. In order to use 'auto' weights, "
-                             "use compute_class_weight\('auto', classes, y\). "
+                             "class_weight 'balanced' is not supported for "
+                             "partial_fit. In order to use 'balanced' weights, "
+                             "use compute_class_weight\('balanced', classes, y\). "
                              "In place of y you can us a large enough sample "
                              "of the full training set target to properly "
                              "estimate the class frequency distributions. "
                              "Pass the resulting weights as the class_weight "
                              "parameter.",
-                             self.factory(class_weight='auto').partial_fit,
+                             self.factory(class_weight='balanced').partial_fit,
                              X, Y, classes=np.unique(Y))
 
     def test_sgd_multiclass(self):
@@ -391,7 +400,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf = self.factory(alpha=0.01, n_iter=20).fit(X2, Y2)
         assert_equal(clf.coef_.shape, (3, 2))
         assert_equal(clf.intercept_.shape, (3,))
-        assert_equal(clf.decision_function([0, 0]).shape, (1, 3))
+        assert_equal(clf.decision_function([[0, 0]]).shape, (1, 3))
         pred = clf.predict(T2)
         assert_array_equal(pred, true_result2)
 
@@ -433,12 +442,12 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf = self.factory(alpha=0.01, n_iter=20, n_jobs=2).fit(X2, Y2)
         assert_equal(clf.coef_.shape, (3, 2))
         assert_equal(clf.intercept_.shape, (3,))
-        assert_equal(clf.decision_function([0, 0]).shape, (1, 3))
+        assert_equal(clf.decision_function([[0, 0]]).shape, (1, 3))
         pred = clf.predict(T2)
         assert_array_equal(pred, true_result2)
 
     def test_set_coef_multiclass(self):
-        # Checks coef_init and intercept_init shape for for multi-class
+        # Checks coef_init and intercept_init shape for multi-class
         # problems
         # Provided coef_ does not match dataset
         clf = self.factory()
@@ -468,16 +477,16 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         # log and modified_huber losses can output probability estimates
         # binary case
         for loss in ["log", "modified_huber"]:
-            clf = self.factory(loss="modified_huber", alpha=0.01, n_iter=10)
+            clf = self.factory(loss=loss, alpha=0.01, n_iter=10)
             clf.fit(X, Y)
-            p = clf.predict_proba([3, 2])
+            p = clf.predict_proba([[3, 2]])
             assert_true(p[0, 1] > 0.5)
-            p = clf.predict_proba([-1, -1])
+            p = clf.predict_proba([[-1, -1]])
             assert_true(p[0, 1] < 0.5)
 
-            p = clf.predict_log_proba([3, 2])
+            p = clf.predict_log_proba([[3, 2]])
             assert_true(p[0, 1] > p[0, 0])
-            p = clf.predict_log_proba([-1, -1])
+            p = clf.predict_log_proba([[-1, -1]])
             assert_true(p[0, 1] < p[0, 0])
 
         # log loss multiclass probability estimates
@@ -489,16 +498,16 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         assert_almost_equal(p[0].sum(), 1)
         assert_true(np.all(p[0] >= 0))
 
-        p = clf.predict_proba([-1, -1])
-        d = clf.decision_function([-1, -1])
+        p = clf.predict_proba([[-1, -1]])
+        d = clf.decision_function([[-1, -1]])
         assert_array_equal(np.argsort(p[0]), np.argsort(d[0]))
 
-        l = clf.predict_log_proba([3, 2])
-        p = clf.predict_proba([3, 2])
+        l = clf.predict_log_proba([[3, 2]])
+        p = clf.predict_proba([[3, 2]])
         assert_array_almost_equal(np.log(p), l)
 
-        l = clf.predict_log_proba([-1, -1])
-        p = clf.predict_proba([-1, -1])
+        l = clf.predict_log_proba([[-1, -1]])
+        p = clf.predict_proba([[-1, -1]])
         assert_array_almost_equal(np.log(p), l)
 
         # Modified Huber multiclass probability estimates; requires a separate
@@ -506,8 +515,8 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         # ordering present in decision_function output.
         clf = self.factory(loss="modified_huber", alpha=0.01, n_iter=10)
         clf.fit(X2, Y2)
-        d = clf.decision_function([3, 2])
-        p = clf.predict_proba([3, 2])
+        d = clf.decision_function([[3, 2]])
+        p = clf.predict_proba([[3, 2]])
         if not isinstance(self, SparseSGDClassifierTestCase):
             assert_equal(np.argmax(d, axis=1), np.argmax(p, axis=1))
         else:   # XXX the sparse test gets a different X2 (?)
@@ -517,9 +526,9 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         # which would cause naive normalization to fail (see comment
         # in SGDClassifier.predict_proba)
         x = X.mean(axis=0)
-        d = clf.decision_function(x)
+        d = clf.decision_function([x])
         if np.all(d < -1):  # XXX not true in sparse test case (why?)
-            p = clf.predict_proba(x)
+            p = clf.predict_proba([x])
             assert_array_almost_equal(p[0], [1 / 3.] * 3)
 
     def test_sgd_l1(self):
@@ -599,20 +608,6 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf = self.factory(alpha=0.1, n_iter=1000, class_weight=[0.5])
         clf.fit(X, Y)
 
-    def test_class_weight_warning(self):
-        """Tests that class_weight passed through fit raises warning.
-           This test should be removed after deprecating support for this"""
-
-        clf = self.factory()
-        warning_message = ("You are trying to set class_weight through the fit "
-                           "method, which is deprecated and will be removed in"
-                           "v0.17 of scikit-learn. Pass the class_weight into "
-                           "the constructor instead.")
-        assert_warns_message(DeprecationWarning,
-                             warning_message,
-                             clf.fit, X4, Y4,
-                             class_weight=1)
-
     def test_weights_multiplied(self):
         # Tests that class_weight and sample_weight are multiplicative
         class_weights = {1: .6, 2: .3}
@@ -629,8 +624,8 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
 
         assert_almost_equal(clf1.coef_, clf2.coef_)
 
-    def test_auto_weight(self):
-        # Test class weights for imbalanced data
+    def test_balanced_weight(self):
+        # Test class weights for imbalanced data"""
         # compute reference metrics on iris dataset that is quite balanced by
         # default
         X, y = iris.data, iris.target
@@ -645,15 +640,16 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         assert_almost_equal(metrics.f1_score(y, clf.predict(X), average='weighted'), 0.96,
                             decimal=1)
 
-        # make the same prediction using automated class_weight
-        clf_auto = self.factory(alpha=0.0001, n_iter=1000,
-                                class_weight="auto", shuffle=False).fit(X, y)
-        assert_almost_equal(metrics.f1_score(y, clf_auto.predict(X), average='weighted'), 0.96,
+        # make the same prediction using balanced class_weight
+        clf_balanced = self.factory(alpha=0.0001, n_iter=1000,
+                                    class_weight="balanced",
+                                    shuffle=False).fit(X, y)
+        assert_almost_equal(metrics.f1_score(y, clf_balanced.predict(X), average='weighted'), 0.96,
                             decimal=1)
 
         # Make sure that in the balanced case it does not change anything
-        # to use "auto"
-        assert_array_almost_equal(clf.coef_, clf_auto.coef_, 6)
+        # to use "balanced"
+        assert_array_almost_equal(clf.coef_, clf_balanced.coef_, 6)
 
         # build an very very imbalanced dataset out of iris data
         X_0 = X[y == 0, :]
@@ -668,14 +664,14 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         y_pred = clf.predict(X)
         assert_less(metrics.f1_score(y, y_pred, average='weighted'), 0.96)
 
-        # fit a model with auto class_weight enabled
-        clf = self.factory(n_iter=1000, class_weight="auto", shuffle=False)
+        # fit a model with balanced class_weight enabled
+        clf = self.factory(n_iter=1000, class_weight="balanced", shuffle=False)
         clf.fit(X_imbalanced, y_imbalanced)
         y_pred = clf.predict(X)
         assert_greater(metrics.f1_score(y, y_pred, average='weighted'), 0.96)
 
         # fit another using a fit parameter override
-        clf = self.factory(n_iter=1000, class_weight="auto", shuffle=False)
+        clf = self.factory(n_iter=1000, class_weight="balanced", shuffle=False)
         clf.fit(X_imbalanced, y_imbalanced)
         y_pred = clf.predict(X)
         assert_greater(metrics.f1_score(y, y_pred, average='weighted'), 0.96)
@@ -718,7 +714,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf.partial_fit(X[:third], Y[:third], classes=classes)
         assert_equal(clf.coef_.shape, (1, X.shape[1]))
         assert_equal(clf.intercept_.shape, (1,))
-        assert_equal(clf.decision_function([0, 0]).shape, (1, ))
+        assert_equal(clf.decision_function([[0, 0]]).shape, (1, ))
         id1 = id(clf.coef_.data)
 
         clf.partial_fit(X[third:], Y[third:])
@@ -737,7 +733,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf.partial_fit(X2[:third], Y2[:third], classes=classes)
         assert_equal(clf.coef_.shape, (3, X2.shape[1]))
         assert_equal(clf.intercept_.shape, (3,))
-        assert_equal(clf.decision_function([0, 0]).shape, (1, 3))
+        assert_equal(clf.decision_function([[0, 0]]).shape, (1, 3))
         id1 = id(clf.coef_.data)
 
         clf.partial_fit(X2[third:], Y2[third:])
@@ -745,6 +741,19 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         # check that coef_ haven't been re-allocated
         assert_true(id1, id2)
 
+    def test_partial_fit_multiclass_average(self):
+        third = X2.shape[0] // 3
+        clf = self.factory(alpha=0.01, average=X2.shape[0])
+        classes = np.unique(Y2)
+
+        clf.partial_fit(X2[:third], Y2[:third], classes=classes)
+        assert_equal(clf.coef_.shape, (3, X2.shape[1]))
+        assert_equal(clf.intercept_.shape, (3,))
+
+        clf.partial_fit(X2[third:], Y2[third:])
+        assert_equal(clf.coef_.shape, (3, X2.shape[1]))
+        assert_equal(clf.intercept_.shape, (3,))
+
     def test_fit_then_partial_fit(self):
         # Partial_fit should work after initial fit in the multiclass case.
         # Non-regression test for #2496; fit would previously produce a
@@ -1025,6 +1034,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase, CommonTest):
                 assert_almost_equal(cd.coef_, sgd.coef_, decimal=2,
                                     err_msg=err_msg)
 
+    @ignore_warnings
     def test_partial_fit(self):
         third = X.shape[0] // 3
         clf = self.factory(alpha=0.01)
@@ -1032,7 +1042,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase, CommonTest):
         clf.partial_fit(X[:third], Y[:third])
         assert_equal(clf.coef_.shape, (X.shape[1], ))
         assert_equal(clf.intercept_.shape, (1,))
-        assert_equal(clf.decision_function([0, 0]).shape, (1, ))
+        assert_equal(clf.predict([[0, 0]]).shape, (1, ))
         id1 = id(clf.coef_.data)
 
         clf.partial_fit(X[third:], Y[third:])
@@ -1151,3 +1161,117 @@ def test_large_regularization():
         with np.errstate(all='raise'):
             model.fit(iris.data, iris.target)
         assert_array_almost_equal(model.coef_, np.zeros_like(model.coef_))
+
+
+def _test_gradient_common(loss_function, cases):
+    # Test gradient of different loss functions
+    # cases is a list of (p, y, expected)
+    for p, y, expected in cases:
+        assert_almost_equal(loss_function.dloss(p, y), expected)
+
+
+def test_gradient_hinge():
+    # Test Hinge (hinge / perceptron)
+    # hinge
+    loss = sgd_fast.Hinge(1.0)
+    cases = [
+        # (p, y, expected)
+        (1.1, 1.0, 0.0), (-2.0, -1.0, 0.0),
+        (1.0, 1.0, -1.0), (-1.0, -1.0, 1.0), (0.5, 1.0, -1.0),
+        (2.0, -1.0, 1.0), (-0.5, -1.0, 1.0), (0.0, 1.0, -1.0)
+    ]
+    _test_gradient_common(loss, cases)
+
+    # perceptron
+    loss = sgd_fast.Hinge(0.0)
+    cases = [
+        # (p, y, expected)
+        (1.0, 1.0, 0.0), (-0.1, -1.0, 0.0),
+        (0.0, 1.0, -1.0), (0.0, -1.0, 1.0), (0.5, -1.0, 1.0),
+        (2.0, -1.0, 1.0), (-0.5, 1.0, -1.0), (-1.0, 1.0, -1.0),
+    ]
+    _test_gradient_common(loss, cases)
+
+
+def test_gradient_squared_hinge():
+    # Test SquaredHinge
+    loss = sgd_fast.SquaredHinge(1.0)
+    cases = [
+        # (p, y, expected)
+        (1.0, 1.0, 0.0), (-2.0, -1.0, 0.0), (1.0, -1.0, 4.0),
+        (-1.0, 1.0, -4.0), (0.5, 1.0, -1.0), (0.5, -1.0, 3.0)
+    ]
+    _test_gradient_common(loss, cases)
+
+
+def test_gradient_log():
+    # Test Log (logistic loss)
+    loss = sgd_fast.Log()
+    cases = [
+        # (p, y, expected)
+        (1.0, 1.0, -1.0 / (np.exp(1.0) + 1.0)),
+        (1.0, -1.0, 1.0 / (np.exp(-1.0) + 1.0)),
+        (-1.0, -1.0, 1.0 / (np.exp(1.0) + 1.0)),
+        (-1.0, 1.0, -1.0 / (np.exp(-1.0) + 1.0)),
+        (0.0, 1.0, -0.5), (0.0, -1.0, 0.5),
+        (17.9, -1.0, 1.0), (-17.9, 1.0, -1.0),
+    ]
+    _test_gradient_common(loss, cases)
+    assert_almost_equal(loss.dloss(18.1, 1.0), np.exp(-18.1) * -1.0, 16)
+    assert_almost_equal(loss.dloss(-18.1, -1.0), np.exp(-18.1) * 1.0, 16)
+
+
+def test_gradient_squared_loss():
+    # Test SquaredLoss
+    loss = sgd_fast.SquaredLoss()
+    cases = [
+        # (p, y, expected)
+        (0.0, 0.0, 0.0), (1.0, 1.0, 0.0), (1.0, 0.0, 1.0),
+        (0.5, -1.0, 1.5), (-2.5, 2.0, -4.5)
+    ]
+    _test_gradient_common(loss, cases)
+
+
+def test_gradient_huber():
+    # Test Huber
+    loss = sgd_fast.Huber(0.1)
+    cases = [
+        # (p, y, expected)
+        (0.0, 0.0, 0.0), (0.1, 0.0, 0.1), (0.0, 0.1, -0.1),
+        (3.95, 4.0, -0.05), (5.0, 2.0, 0.1), (-1.0, 5.0, -0.1)
+    ]
+    _test_gradient_common(loss, cases)
+
+
+def test_gradient_modified_huber():
+    # Test ModifiedHuber
+    loss = sgd_fast.ModifiedHuber()
+    cases = [
+        # (p, y, expected)
+        (1.0, 1.0, 0.0), (-1.0, -1.0, 0.0), (2.0, 1.0, 0.0),
+        (0.0, 1.0, -2.0), (-1.0, 1.0, -4.0), (0.5, -1.0, 3.0),
+        (0.5, -1.0, 3.0), (-2.0, 1.0, -4.0), (-3.0, 1.0, -4.0)
+    ]
+    _test_gradient_common(loss, cases)
+
+
+def test_gradient_epsilon_insensitive():
+    # Test EpsilonInsensitive
+    loss = sgd_fast.EpsilonInsensitive(0.1)
+    cases = [
+        (0.0, 0.0, 0.0), (0.1, 0.0, 0.0), (-2.05, -2.0, 0.0),
+        (3.05, 3.0, 0.0), (2.2, 2.0, 1.0), (2.0, -1.0, 1.0),
+        (2.0, 2.2, -1.0), (-2.0, 1.0, -1.0)
+    ]
+    _test_gradient_common(loss, cases)
+
+
+def test_gradient_squared_epsilon_insensitive():
+    # Test SquaredEpsilonInsensitive
+    loss = sgd_fast.SquaredEpsilonInsensitive(0.1)
+    cases = [
+        (0.0, 0.0, 0.0), (0.1, 0.0, 0.0), (-2.05, -2.0, 0.0),
+        (3.05, 3.0, 0.0), (2.2, 2.0, 0.2), (2.0, -1.0, 5.8),
+        (2.0, 2.2, -0.2), (-2.0, 1.0, -5.8)
+    ]
+    _test_gradient_common(loss, cases)
